#include "img.h"
#include "libmy/mode3.h"
#include "libmy/fade.h"
#include "file.h"

//---------------------------------------------------------------------------
ST_IMG Img;


//---------------------------------------------------------------------------
EWRAM_CODE void ImgInit(void)
{
	_Memset(&Img, 0x00, sizeof(ST_IMG));
}
//---------------------------------------------------------------------------
IWRAM_CODE void ImgExec(void)
{
	if(Img.isDraw == FALSE)
	{
		return;
	}


	switch(Img.effectNo)
	{
	case IMG_EFFECT_UD:
		ImgExecEffectUd();
		break;

	case IMG_EFFECT_MASK:
	case IMG_EFFECT_ALPHA:		// p
		ImgExecEffectMask();
		break;

	case IMG_EFFECT_LR:
		ImgExecEffectLr();
		break;

	case IMG_EFFECT_NORMAL:
	default:
		ImgExecEffectNormal();
		break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void ImgExecBuf(void)
{
	if(Img.bgStr[0] != '\0')
	{
		ImgDrawBg(Img.bgStr);
	}
	else
	{
		Mode3DrawBgClear();
	}

	u32 i;

	for(i=0; i<IMG_MAX_CHR_CNT; i++)
	{
		if(Img.chrStr[i][0] != '\0')
		{
			ImgDrawChr(Img.chrStr[i], Img.chrPos[i]);
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void ImgExecEffectUd(void)
{
	switch(Img.effectStep)
	{
	case 0:
		ImgExecBuf();
		Img.effectStep++;
		break;

	default: ; 
		u32 y = ((Img.effectStep-1) * 8);
		Mode3Copy(0, y, 239, y+7);
		Img.effectStep++;

		if(Img.effectStep >= 21)
		{
			Img.isDraw = FALSE;
		}
		break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void ImgExecEffectLr(void)
{
	switch(Img.effectStep)
	{
	case 0:
		ImgExecBuf();
		Img.effectStep++;
		break;

	default: ; 
		u32 x = ((Img.effectStep-1) * 8);
		Mode3Copy(x, 0, x+7, 159);
		Img.effectStep++;

		if(Img.effectStep >= 31)
		{
			Img.isDraw = FALSE;
		}
		break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void ImgExecEffectMask(void)
{
	u32 cx[4] = { 0, 3, 0, 3 };
	u32 cy[4] = { 0, 3, 3, 0 };
	u32 x;
	u32 y;
	u32 idx = Img.effectStep - 1;

	switch(Img.effectStep)
	{
	case 0:
		ImgExecBuf();
		Img.effectStep++;
		break;

	default:
		for(y=0; y<160; y+=8)
		{
			for(x=0; x<240; x+=8)
			{
				Mode3Copy(x+cx[idx], y+cy[idx], x+cx[idx]+4, y+cy[idx]+4);
			}
		}

		if(idx >= 3)
		{
			Img.isDraw = FALSE;
		}
		Img.effectStep++;
		break;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void ImgExecEffectNormal(void)
{
	switch(Img.effectStep)
	{
	case 0:
		ImgExecBuf();
		Img.effectStep++;
		break;

	case 1:
		Mode3SetDraw(TRUE);
		Img.isDraw = FALSE;
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgDrawBg(char* pStr)
{
	ST_IMG_HEADER* p = (ST_IMG_HEADER*)FileGetImg(pStr);

	Mode3DrawBg((u16*)(p + 1));
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgDrawChr(char* pStr, u32 pos)
{
	ST_IMG_HEADER* p = (ST_IMG_HEADER*)FileGetImg(pStr);
	u32 sx;

	if(pos == 0)
	{
		sx = 10;
	}
	else if (pos == 2)
	{
		sx = SCREEN_WIDTH - p->cx - 10;
	}
	else
	{
		sx = (SCREEN_WIDTH / 2) - (p->cx / 2);
	}

	Mode3DrawImgMask(sx, 0, p->cx, p->cy, (u16*)(p + 1), RGB5(0, 0, 0));
}
//---------------------------------------------------------------------------
// J_[̏ݎɎg܂
EWRAM_CODE void ImgDrawCircle(u32 sx, u32 sy)
{
	u32 i;

	for(i=0; i<3; i++)
	{
		Mode3DrawCircle(sx, sy, 10+i, RGB5(31, 0, 0));
	}

	Mode3SetDraw(TRUE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetBg(char* pStr)
{
	_Strncpy(Img.bgStr, pStr, IMG_MAX_STR_SIZE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetBgClear(void)
{
	ImgSetBg("");
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetChr(char* pStr, u32 no, u32 pos)
{
	_ASSERT(no  < IMG_MAX_CHR_CNT);
	_ASSERT(pos < IMG_MAX_POS_CNT);

	_Strncpy(Img.chrStr[no], pStr, IMG_MAX_STR_SIZE);
	Img.chrPos[no] = pos;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetChrClear(void)
{
	ImgSetChr("", 0, 0);
	ImgSetChr("", 1, 0);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetEffect(u32 no)
{
	_ASSERT(no < IMG_EFFECT_END);

	TRACEOUT("effect: %d\n", no);

	Img.effectNo = no;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ImgSetDraw(void)
{
	Img.isDraw = TRUE;
	Img.effectStep = 0;
}
//---------------------------------------------------------------------------
// fobOp
EWRAM_CODE void ImgDirect(char* pStr)
{
	ImgDrawBg(pStr);
	Mode3SetDraw(TRUE);
}
//---------------------------------------------------------------------------
EWRAM_CODE bool ImgIsEnd(void)
{
	return (Img.isDraw == FALSE) ? TRUE : FALSE;
}
