#include "mode3.h"
#include "bios_arm.h"
#include "../res.h"

//---------------------------------------------------------------------------
ST_MODE3 Mode3 EWRAM_BSS;


//---------------------------------------------------------------------------
EWRAM_CODE void Mode3Init(void)
{
	BiosExecFixClear(&Mode3, sizeof(ST_MODE3));

	REG_DISPCNT = MODE_3 | BG2_ON;
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3Exec(void)
{
	if(Mode3.isDraw == TRUE)
	{
		BiosExecFast(Mode3.bg, (void*)VRAM, MODE3_MAX_BUF_SIZE);
		Mode3.isDraw = FALSE;
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBg(u16* pBuf)
{
	LZ77UnCompVram(pBuf, Mode3.bg);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBgFill(u16 col)
{
	u32 c =((u32)col << 16) | (u32)col;

	BiosExecFastFix(&c, Mode3.bg, MODE3_MAX_BUF_SIZE);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawBgClear(void)
{
	Mode3DrawBgFill(RGB5(0, 0, 0));
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawImg(u32 sx, u32 sy, u32 cx, u32 cy, u16* img)
{
	LZ77UnCompVram(img, Mode3.chr);


	u16* p = Mode3.bg + (sy * SCREEN_WIDTH) + sx;
	u32  x, y;

	for(y=0; y<cy; y++)
	{
		for(x=0; x<cx; x++)
		{
			p[y*SCREEN_WIDTH + x] = Mode3.chr[y*cx + x];
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawImgMask(u32 sx, u32 sy, u32 cx, u32 cy, u16* img, u16 mask)
{
	LZ77UnCompVram(img, Mode3.chr);


	u16* p = Mode3.bg + (sy * SCREEN_WIDTH) + sx;
	u32  x, y;

	for(y=0; y<cy; y++)
	{
		for(x=0; x<cx; x++)
		{
			if(Mode3.chr[y*cx + x] == mask)
			{
				continue;
			}

			p[y*SCREEN_WIDTH + x] = Mode3.chr[y*cx + x];
		}
	}
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3Copy(u32 sx, u32 sy, u32 ex, u32 ey)
{
	u32 x, y;

	for(y=sy; y<=ey; y++)
	{
		for(x=sx; x<=ex; x++)
		{
			((u16*)VRAM)[(y * SCREEN_WIDTH) + x] = Mode3.bg[(y * SCREEN_WIDTH) + x];
		}
	}

//	_ASSERT(ex < SCREEN_WIDTH);
//	_ASSERT(ey < SCREEN_HEIGHT);
}
//---------------------------------------------------------------------------
IWRAM_CODE void Mode3DrawCircle(u32 sx, u32 sy, u32 rad, u16 col)
{
	u32 x = rad;
	u32 y = 0;
	s32 p = 3 - 2 * rad;

	while(x >= y)
	{
		Mode3.bg[((sy + x) * SCREEN_WIDTH) + (sx + y)] = col;
		Mode3.bg[((sy - x) * SCREEN_WIDTH) + (sx + y)] = col;
		Mode3.bg[((sy + x) * SCREEN_WIDTH) + (sx - y)] = col;
		Mode3.bg[((sy - x) * SCREEN_WIDTH) + (sx - y)] = col;
		Mode3.bg[((sy + y) * SCREEN_WIDTH) + (sx + x)] = col;
		Mode3.bg[((sy - y) * SCREEN_WIDTH) + (sx + x)] = col;
		Mode3.bg[((sy + y) * SCREEN_WIDTH) + (sx - x)] = col;
		Mode3.bg[((sy - y) * SCREEN_WIDTH) + (sx - x)] = col;

		if(p >= 0)
		{
			x--;
			p -= 4 * x;
		}

		y++;
		p += 4 * y + 2;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void Mode3SetDraw(bool isDraw)
{
	Mode3.isDraw = isDraw;
}
