#include "nv.h"
#include "nv2.h"
#include "libmy/key.h"
#include "file.h"
#include "history.h"
#include "img.h"
#include "menu.h"
#include "bgm.h"


//---------------------------------------------------------------------------
ST_NV Nv;


//---------------------------------------------------------------------------
EWRAM_CODE void NvInit(void)
{
	_Memset(&Nv, 0x00, sizeof(ST_NV));

	NvSetSnx("INIT");
	NvSetAct(NV_ACT_PARSE);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvInitVar(void)
{
	_Memset(&Nv.varSystem, 0x00, sizeof(NV_MAX_VAR_CNT));
	_Memset(&Nv.varSwitch, 0x00, sizeof(NV_MAX_VAR_CNT));
	_Memset(&Nv.varCount,  0x00, sizeof(NV_MAX_VAR_CNT));
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExec(void)
{
	if(ImgIsEnd() == FALSE || TextIsEnd() == FALSE || HistoryIsEnd() == FALSE || MenuIsEnd() == FALSE)
	{
		return;
	}


	switch(Nv.act)
	{
	case NV_ACT_PARSE:
		NvExecParse();
		break;

	case NV_ACT_KEY:
		NvExecKey();
		break;

	case NV_ACT_KEY_WAIT:
		NvExecKeyWait();
		break;

	case NV_ACT_CALENDAR:
		NvExecCalendar();
		break;

	case NV_ACT_SELECT:
		NvExecSelect();
		break;

	default:
		SystemError("[Err] NvExec Nv.act = %d\n", Nv.act);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecKey(void)
{
	if(NvExecKeySub() == TRUE)
	{
		TextSetLf();
		NvSetAct(NV_ACT_PARSE);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE bool NvExecKeySub(void)
{
	u16 cnt = KeyGetCnt();
	u16 trg = KeyGetTrg();

	TextShowCur();

	if(Nv.isSkip == TRUE)
	{
		if(cnt & KEY_B)
		{
			Nv.isSkip = FALSE;
		}
		else
		{
			TextHideCur();
			return TRUE;
		}
	}

	if((cnt & KEY_R) || (trg & KEY_A) || (trg & KEY_DOWN))
	{
		TextHideCur();
		return TRUE;
	}
	else if((trg & KEY_LEFT) && HistoryIsEmpty() == FALSE)
	{
		TextHideCur();
		HistoryInitAct(TRUE);

		return FALSE;
	}
	else if(trg & KEY_B)
	{
		TextHideCur();
		MenuSetSystem();

		return FALSE;
	}

	return FALSE;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecKeyWait(void)
{
	u16 trg = KeyGetTrg();

	if((trg & KEY_A) || (trg & KEY_DOWN))
	{
		NvSetAct(NV_ACT_PARSE);
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecCalendar(void)
{
	static const s32 daypad[] = {0, 4, 0, 0, 3, 0, 0, 0, 0, 0, 0, -1, 1};
	static const s32 xsub[] = {0, 0, 0, -1, -1, -2, -2};
	static const char* const calFile[] = {"", "ETC18", "ETC19", "ETC20", "ETC21", "", "", "", "", "", "", "ETC16", "ETC17"};

	switch(Nv.actStep)
	{
	case 0:
		TextHide();
		TextClear();

		ImgSetBg((char*)calFile[ NvGetVarCount(0) ]);
		ImgSetEffect(IMG_EFFECT_MASK);
		ImgSetDraw();

		Nv.actStep++;
		break;

	case 1: ;
		s32 n  = daypad[ NvGetVarCount(0) ] + NvGetVarCount(1);
		u32 sx = 43 + Mod(n, 7) * 26 + xsub[ Mod(n, 7) ];
		u32 sy = 52 + Div(n, 7) * 20;

		ImgDrawCircle(sx, sy);
		Nv.actStep++;
		break;

	case 2: ; 
		u16 trg = KeyGetTrg();
		u16 cnt = KeyGetCnt();

		if((cnt & KEY_R) || (trg & KEY_A) || (trg & KEY_DOWN) || (Nv.isSkip == TRUE))
		{
			Nv.actStep++;
		}
		break;

	case 3:
		ImgSetBgClear();
		ImgSetEffect(IMG_EFFECT_MASK);
		ImgSetDraw();

		Nv.actStep++;
		break;

	case 4:
		NvSetAct(NV_ACT_PARSE);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvExecSelect(void)
{
	s32 i;

	switch(Nv.actStep)
	{
	case 0:
		TextClear();
		TextShow();
		Nv.actStep++;
		break;

	case 1:
		Nv.sel = -1;
		Nv.actStep++;
		break;

	case 2:
		for(i=0; i<Nv.selCnt; i++)
		{
			if(Nv.sel == i)
			{
				TextDrawStrXy(0, i, (u16*)Nv.selStr[i]);
			}
			else
			{
				TextDrawStrXyGray(0, i, (u16*)Nv.selStr[i]);
			}
		}
		Nv.actStep++;
		break;

	case 3: ;
		u16 trg = KeyGetTrg();

		if((trg & KEY_UP) && Nv.sel > 0)
		{
			Nv.sel--;
			Nv.actStep--;
		}
		else if((trg & KEY_DOWN) && (Nv.sel + 1) < Nv.selCnt)
		{
			Nv.sel++;
			Nv.actStep--;
		}
		else if((trg & KEY_A) && Nv.sel != -1)
		{
			Nv.actStep++;
		}
		else if((trg & KEY_LEFT) && HistoryIsEmpty() == FALSE)
		{
			HistoryInitAct(TRUE);
			Nv.actStep--;
		}
		else if(trg & KEY_B)
		{
			MenuSetSystem();
			Nv.actStep--;
		}
		break;

	case 4:
		NvSetVarCount(Nv.selIdx, Nv.sel + 1);
		Nv.isSkip = FALSE;
		TextClearDat();

		NvSetAct(NV_ACT_PARSE);
		break;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetAct(u32 act)
{
	Nv.act = act;
	Nv.actStep = 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetSnx(char* pStr)
{
	u8* p = (u8*)FileGetSnx(pStr);

	Nv.pHead = p;
	Nv.pCur  = p;
	Nv.size  = FileGetSize();
	_Strncpy(Nv.str, pStr, NV_MAX_STR_SIZE);

	TRACEOUT("[Load: str:%s]\n", Nv.str);
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetSkip(bool is)
{
	Nv.isSkip = is;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetVarSystem(u32 idx, u8 val)
{
	_ASSERT(idx < NV_MAX_VAR_CNT);
	TRACEOUT("[SetSy[%d] = %d]\n", idx, val);

	Nv.varSystem[idx] = val;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetVarSwitch(u32 idx, u8 val)
{
	_ASSERT(idx < NV_MAX_VAR_CNT);
	TRACEOUT("[SetSw[%d] = %d]\n", idx, val);

	Nv.varSwitch[idx] = val;
}
//---------------------------------------------------------------------------
EWRAM_CODE void NvSetVarCount(u32 idx, u8 val)
{
	_ASSERT(idx < NV_MAX_VAR_CNT);
	TRACEOUT("[SetCo[%d] = %d]\n", idx, val);

	Nv.varCount[idx] = val;
}
//---------------------------------------------------------------------------
EWRAM_CODE u8 NvGetVarSystem(u32 idx)
{
	_ASSERT(idx < NV_MAX_VAR_CNT);
	TRACEOUT("[GetSy[%d] val:%d]\n", idx, Nv.varSystem[idx]);

	return Nv.varSystem[idx];
}
//---------------------------------------------------------------------------
EWRAM_CODE u8 NvGetVarSwitch(u32 idx)
{
	_ASSERT(idx < NV_MAX_VAR_CNT);
	TRACEOUT("[GetSw[%d] val:%d]\n", idx, Nv.varSwitch[idx]);

	return Nv.varSwitch[idx];
}
//---------------------------------------------------------------------------
EWRAM_CODE u8 NvGetVarCount(u32 idx)
{
	_ASSERT(idx < NV_MAX_VAR_CNT);
	TRACEOUT("[GetCo[%d] val:%d]\n", idx, Nv.varCount[idx]);

	return Nv.varCount[idx];
}
//---------------------------------------------------------------------------
EWRAM_CODE bool NvIsEnd(void)
{
	return (Nv.act == NV_ACT_END) ? TRUE : FALSE;
}
