#include <stdio.h>
#include <string.h>
#include "onesnx.h"


//---------------------------------------------------------------------------

const char* FileList[] = {
	// index = 492
	"INIT",
	"AK21",
	"AK22",
	"AK24",
	"AK25",
	"AK26",
	"AK27",
	"AK27_B",
	"AK_END",
	"AKANEN",
	"AKMI21",
	"AKMI22",
	"AP00",
	"AP01",
	"ASA",
	"DS01",
	"DS02",
	"DS03",
	"DS04",
	"DS05",
	"DS07",
	"DS08",
	"DS09",
	"DS10",
	"DS10_A",
	"DS11",
	"DS14",
	"DS15",
	"DS16",
	"DS17",
	"DS18",
	"DS19",
	"DS22",
	"DS24",
	"DS25",
	"HIKAMI",
	"MI_A",
	"MI21",
	"MI22",
	"MI24",
	"MI25",
	"MI26",
	"MIMS21",
	"MISAO",
	"MS_B",
	"MS21",
	"MS22",
	"MS24",
	"MS25",
	"MS26",
	"MS27",
	"MS28",
	"MY21",
	"MY22",
	"MY24",
	"MY25",
	"MZ19",
	"MZ21",
	"MZ22",
	"MZ24",
	"MZ25~",
	"MZ26",
	"NV30",
	"RM14",
	"RM15",
	"RM16",
	"RM17",
	"RM18",
	"RM19",
	"RM21",
	"RM22",
	"RM24",
	"RM25",
	"RM26",
	"RMM1",
	"RMM2",
	"RMM3",
	"WORLD",
	"DEBUG",
};

//---------------------------------------------------------------------------
int main()
{
	u8* pSrc;
	u8* pDst;
	u32 srcSize;
	u32 dstSize;

	s32 i;

	for(i=0; i<ARRAY_SIZE(FileList); i++)
	{
		printf("decode snx ... %s\n", FileList[i]);

		srcSize = OpenBuf(FileList[i], &pSrc);
		dstSize = DecodeSnx(pSrc, &pDst);

		SaveFile(FileList[i], pDst, dstSize);

		free(pSrc);
		free(pDst);
	}

	return 0;
}
//---------------------------------------------------------------------------
u32 OpenBuf(const char* pName, u8** pBuf)
{
	FILE* fp = fopen(pName, "rb");

	if(fp == NULL)
	{
		printf("err filename\n");
		exit(1);
	}

	fpos_t fsize;
	fseek(fp,0,SEEK_END);
	fgetpos(fp, &fsize);
	fseek(fp,0,SEEK_SET);

	*pBuf = (u8*)calloc(fsize, sizeof(u8));
	fread(*pBuf, sizeof(u8), fsize, fp);
	fclose(fp);

	return fsize;
}
//---------------------------------------------------------------------------
u32 DecodeSnx(u8* pSrc, u8** pDst)
{
	unsigned long *size;
	unsigned char *ret, *ptr, *src, bit, *endptr;
	int tree_size, tree_id, left, right, mid;

	ScriptFileHeader* header;
	HuffmanTree *tree, *basetree;

	// $BE83+8e$N%G!<%?$r3JG<$9$k%a%b%j$r3NJ](B
	header = (ScriptFileHeader*)pSrc;
	basetree = (HuffmanTree *)(header + 1);
	*size = header->expandSize;

	ret = (unsigned char*)calloc(*size, sizeof(u8));

	if(ret == NULL)
	{
		printf("err snx file\n");
		exit(1);
	}

	tree_size = (header->rootID & 0xff) + 1 + header->numLeaf;
	src = (unsigned char*)(header + 1 + tree_size);
	bit = 0x80;
	ptr = ret;
	endptr = ret+(*size);

	while (ptr<endptr) {
		tree_id = header->rootID;
		do {
			// $BFsJ,8!:w$G(BID$B$N0lCW$9$kLZ$rC5$9(B
			left = 0;
			right = tree_size;
			while (left<right) {
				mid = (left + right) / 2;
				if ( basetree[mid].id<tree_id ) {
					left = mid + 1;
				} else {
					right = mid;
				}
			}
			tree = &basetree[left];

			// 1bit$BFI$_9~$_(B, $BLZ$r2<$X9_$j$F$$$/(B
			tree_id = ((*src)&bit)?tree->child1:tree->child0;
			bit >>= 1;
			if ( !bit ) {
				src++;
				bit = 0x80;
				// $BFI$_9~$_=*N;%A%'%C%/$r$9$k$J$i$3$3$G9T$J$&(B
			}
		} while (tree_id & 0xff00);

		// $B8=:_$N(Btree_id$B$,$=$N$^$^J8;z%3!<%I$K$J$k(B
		*ptr++ = tree_id;
	}

	*pDst = ret;
	return *size;
}
//---------------------------------------------------------------------------
void SaveFile(const char* pName, u8* pDst, u32 dstSize)
{
	char dstName[64];
	strcpy(dstName, pName);
	strcat(dstName, ".SNX");

	FILE* out = fopen(dstName, "wb");
	if(out == NULL)
	{
		printf("err save file: %s\n", dstName);
		exit(1);
	}

	fwrite(pDst, sizeof(u8), dstSize, out);
	fclose(out);
}
