/*=======================================================================
  xakane for Zaurus
  Copyright 2001, K.Takagi(Kenjo)
=======================================================================*/
/* ================================================================
   ***** Original Code Copyright ******
 
   xakane  --- ONE ` for bright season for X ---
   (c) Copyright 2000, 2001 Nao <nao@sore.totto.to>
   All rights reserved.
================================================================ */

#include <zheaders.h>
#include <sound.h>
#include "wlk1_C_usr.h"
#include "system.h"
#include "soundmgr.h"

static char SoundBaseDir[64];
static char CD_LoopFlag;
static char Volume;
static int CD_FadeTime;
static unsigned int CD_FadeTimeBase;
static short CurTrack;

static SndParam *param;
static DRFFILE *drf;

static char Sound_MasterVol = 1;

//#define SOUNDDELAYFIX		// ł̍ĐJn܂ł̃fBCCꍇON
#define SOUNDDELAY 12		// ĐJñfBC

/************************************************************************
  class SOUND
    TEh֘A
************************************************************************/

void Sound_Init(char* drv)
{
	sprintf(SoundBaseDir, "%s:\\ONE\\WLK1CD", drv);

	Volume = Sound_MasterVol;
	CurTrack = -1;
	CD_LoopFlag = 0;
	CD_FadeTime = 0;
	CD_FadeTimeBase = 0;
	param = 0;
	drf = 0;
}

void Sound_Cleanup(void)
{
	Sound_CD_Stop();
	Wait100mSec(5);
}


void Sound_Reset(void)
{
	Sound_CD_Stop();
	CurTrack = -1;
	CD_LoopFlag = 0;
	CD_FadeTime = 0;
	CD_FadeTimeBase = 0;
}


/* -------------------------------------------------------------------
  CD-DAĐ (Internal routine)
------------------------------------------------------------------- */
static void Sound_CDDA_Play(short trk)
{
	int i, n = -1;
	char file[64];

	if ( Sound_MasterVol==-1 ) return;

	sprintf(file, "%s%02d.WAV", SoundBaseDir, trk);
	param = ZsdCreateSndParam();
	if ( !param ) {
		return;
	}
	drf = ZsdDRFfopen(file, "r");
	if ( drf ) {
		ZsdGetCond(drf, param->zsd);
		DA_GAIN(7-Volume);
		ZsdCont(param);
		ZsdPlayDRF(param, drf, 0, ZSD_SIZE_ALL);
		for (i=0; (i<10)&&(param->endFlag!=SNDPROC_RUNNING); i++ )
			Wait100mSec(1);
		if ( i==10 ) {
			do {
				ZsdStop(param);
				Wait100mSec(1);
			} while ( param->endFlag==SNDPROC_RUNNING );
			ZsdDRFfclose(drf);
			ZsdDisposeSndParam(param);
			param = 0;
			drf = 0;
		}
	} else {
		ZsdDisposeSndParam(param);
		param = 0;
		drf = 0;
	}
}


static void Sound_CDDA_Stop(void)
{
	if ( param ) {
		while ( param->endFlag==SNDPROC_RUNNING ) {
			ZsdStop(param);
			Wait100mSec(1);
		}
	}
	if ( drf )
		ZsdDRFfclose(drf);
	if ( param )
		ZsdDisposeSndParam(param);
	param = 0;
	drf = 0;
	Volume = Sound_MasterVol;
	Wait100mSec(1);
}


static void Sound_CDDA_SetVolume(int vol)
{
	Volume = vol;
	if ( Sound_MasterVol==-1 ) return;
	DA_GAIN(7-Volume);
}


/* -------------------------------------------------------------------
  CD/DSoundĐ JI/F
------------------------------------------------------------------- */
void Sound_CD_Play(short trk, short loop)
{
	Sound_CDDA_Stop();
	Sound_CDDA_Play(trk);
	Sound_CDDA_SetVolume(Sound_MasterVol);
	CD_LoopFlag = loop;
	CD_FadeTime = 0;
	CurTrack = trk;
}


void Sound_CD_Stop(void)
{
	Sound_CDDA_Stop();
	CD_LoopFlag = 0;
	CD_FadeTime = 0;
	CurTrack = -1;
}


void Sound_CD_FadeOut(int n)
{
	if ( Sound_MasterVol==-1 ) return;
	CD_FadeTime = n*40;
	CD_FadeTimeBase = Sys_GetCurrentTimer();
}


void Sound_CD_MasterVol(char vol)
{
	if ( (Sound_MasterVol==-1)&&(vol!=-1) ) {
		Sound_MasterVol = vol;
		Sound_CDDA_Play(CurTrack);
		Sound_CDDA_SetVolume(Sound_MasterVol);
	} else if ( (Sound_MasterVol!=-1)&&(vol==-1) ) {
		Sound_MasterVol = vol;
		Sound_CDDA_Stop();
	} else if ( Sound_MasterVol!=-1 ) {
		Volume = Sound_MasterVol = vol;
		DA_GAIN(7-vol);
	}
}


void Sound_CD_CheckPlay(void)
{
	int t, l;

	if ( CD_FadeTime>0 ) {				// tF[hAEg
		t = 1000-(((Sys_GetCurrentTimer()-CD_FadeTimeBase)*1000)/(CD_FadeTime*2));	// CĂ
		if ( (t<0)||(t>1000) ) {
			Sound_CDDA_Stop();
			CD_FadeTime = 0;
			return;
		}
		l = (Sound_MasterVol*t)/1000;
		Sound_CDDA_SetVolume(l);
	} else if ( CD_FadeTime<0 ) {		// ̎̓tF[hC
		t = (((Sys_GetCurrentTimer()-CD_FadeTimeBase)*1000)/(-CD_FadeTime*2));
		if ( (t<0)||(t>1000) ) {
			t = 1000;
			CD_FadeTime = 0;
		}
		l = (Sound_MasterVol*t)/1000;
		Sound_CDDA_SetVolume(l);
	}

	if ( !CD_LoopFlag ) return;		// [vtł͂Ȃ
	if ( !param ) return;		// tł͂Ȃ

	if ( param->endFlag!=SNDPROC_RUNNING ) {
		Sound_CDDA_Stop();
		Sound_CDDA_Play(CurTrack);
		Sound_CDDA_SetVolume(Sound_MasterVol);
	}
}
